﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Management;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the installed DIMM modules.
    /// </summary>
    public struct DIMM
    {
        /// <summary>
        /// Enumerates all installed DIMMs.
        /// </summary>
        public static DIMM[] EnumerateDIMMs()
        {
            ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\CIMV2", "SELECT * FROM Win32_PhysicalMemory");
            List<DIMM> dList = new List<DIMM>();
            foreach (ManagementObject queryObj in searcher.Get())
            {
                DIMM info = new DIMM();
                try { info.Bank = queryObj["BankLabel"].ToString(); }
                catch { }

                try { info.Capacity = Convert.ToUInt64(queryObj["Capacity"].ToString()); }
                catch { }

                try { info.Caption = queryObj["Caption"].ToString(); }
                catch { }

                try { info.DataWidth = Convert.ToInt32(queryObj["DataWidth"].ToString()); }
                catch { }

                try { info.Description = queryObj["Description"].ToString(); }
                catch { }

                try { info.DeviceLocator = queryObj["DeviceLocator"].ToString(); }
                catch { }

                try { info.FormFactor = Convert.ToInt32(queryObj["FormFactor"].ToString()); }
                catch { }

                try { info.InterleaveDataDepth = Convert.ToInt32(queryObj["InterleaveDataDepth"].ToString()); }
                catch { }

                try { info.InterleavePosition = Convert.ToInt32(queryObj["InterleavePosition"].ToString()); }
                catch { }

                try { info.IsHotSwappable = Convert.ToBoolean(queryObj["HotSwappable"].ToString()); }
                catch { }

                try { info.IsPoweredOn = Convert.ToBoolean(queryObj["PoweredOn"].ToString()); }
                catch { }

                try { info.IsRemovable = Convert.ToBoolean(queryObj["Removable"].ToString()); }
                catch { }

                try { info.IsReplaceable = Convert.ToBoolean(queryObj["Replaceable"].ToString()); }
                catch { }

                try { info.Manufacturer = queryObj["Manufacturer"].ToString(); }
                catch { }

                try { info.MemoryType = Convert.ToInt32(queryObj["MemoryType"].ToString()); }
                catch { }

                try { info.Model = queryObj["Model"].ToString(); }
                catch { }

                try { info.Name = queryObj["Name"].ToString(); }
                catch { }

                try { info.PartNumber = queryObj["PartNumber"].ToString(); }
                catch { }

                try { info.PositionInRow = Convert.ToInt32(queryObj["PositionInRow"].ToString()); }
                catch { }

                try { info.SerialNumber = queryObj["SerialNumber"].ToString(); }
                catch { }

                try { info.SKU = queryObj["SKU"].ToString(); }
                catch { }

                try { info.Speed = Convert.ToInt32(queryObj["Speed"].ToString()); }
                catch { }

                try { info.Status = queryObj["Status"].ToString(); }
                catch { }

                try { info.Tag = queryObj["Tag"].ToString(); }
                catch { }

                try { info.TotalWidth = Convert.ToInt32(queryObj["TotalWidth"].ToString()); }
                catch { }

                try { info.TypeDetail = Convert.ToInt32(queryObj["TypeDetail"].ToString()); }
                catch { }

                try { info.Version = queryObj["Version"].ToString(); }
                catch { }

                dList.Add(info);
            }
            return dList.ToArray();
        }

        /// <summary>
        /// The bank.
        /// </summary>
        public string Bank { get; set; }
        /// <summary>
        /// The capacity.
        /// </summary>
        public ulong Capacity { get; set; }
        /// <summary>
        /// The caption.
        /// </summary>
        public string Caption { get; set; }
        /// <summary>
        /// The data width.
        /// </summary>
        public int DataWidth { get; set; }
        /// <summary>
        /// The description.
        /// </summary>
        public string Description { get; set; }
        /// <summary>
        /// The device locator.
        /// </summary>
        public string DeviceLocator { get; set; }
        /// <summary>
        /// The form factor.
        /// </summary>
        public int FormFactor { get; set; }
        /// <summary>
        /// Information, whether it is hot swappable.
        /// </summary>
        public bool IsHotSwappable { get; set; }
        /// <summary>
        /// The interleave data depth.
        /// </summary>
        public int InterleaveDataDepth { get; set; }
        /// <summary>
        /// The interleave position.
        /// </summary>
        public int InterleavePosition { get; set; }
        /// <summary>
        /// The manufacturer.
        /// </summary>
        public string Manufacturer { get; set; }
        /// <summary>
        /// The memory type.
        /// </summary>
        public int MemoryType { get; set; }
        /// <summary>
        /// The model.
        /// </summary>
        public string Model { get; set; }
        /// <summary>
        /// The name.
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// The part number.
        /// </summary>
        public string PartNumber { get; set; }
        /// <summary>
        /// The position in row.
        /// </summary>
        public int PositionInRow { get; set; }
        /// <summary>
        /// Information, whether it is powered on.
        /// </summary>
        public bool IsPoweredOn { get; set; }
        /// <summary>
        /// Information, whether it is removable. 
        /// </summary>
        public bool IsRemovable { get; set; }
        /// <summary>
        /// Information, whether it is replaceable.
        /// </summary>
        public bool IsReplaceable { get; set; }
        /// <summary>
        /// The serial number.
        /// </summary>
        public string SerialNumber { get; set; }
        /// <summary>
        /// The SKU.
        /// </summary>
        public string SKU { get; set; }
        /// <summary>
        /// The speed.
        /// </summary>
        public int Speed { get; set; }
        /// <summary>
        /// The status.
        /// </summary>
        public string Status { get; set; }
        /// <summary>
        /// The tag.
        /// </summary>
        public string Tag { get; set; }
        /// <summary>
        /// The total width.
        /// </summary>
        public int TotalWidth { get; set; }
        /// <summary>
        /// The type detail.
        /// </summary>
        public int TypeDetail { get; set; }
        /// <summary>
        ///  The version.
        /// </summary>
        public string Version { get; set; }

        /// <summary>
        /// Returns true if two objects are inequal, otherwise false.
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <returns></returns>
        public static bool operator !=(DIMM x, DIMM y)
        {
            bool result = false;
            if (x.Bank != y.Bank)
            { result = true; }
            return result;
        }

        /// <summary>
        /// Compares the current object with another object.
        /// </summary>
        /// <returns></returns>
        public override bool Equals(Object obj)
        {
            DIMM dimm;
            try { dimm = (DIMM)obj; }
            catch { return false; }

            if (this.Bank == dimm.Bank)
            { return true; }
            else
            { return false; }
        }

        /// <summary>
        /// Returns true if two objects are equal, otherwise false.
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <returns></returns>
        public static bool operator ==(DIMM x, DIMM y)
        {
            bool result = false;
            if (x.Bank == y.Bank)
            { result = true; }
            return result;
        }

        /// <summary>
        /// Returns the hash code for this instance.
        /// </summary>
        /// <returns></returns>
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }
}
