﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Win32;

namespace TweakAPI.Tweak
{
    /// <summary>
    /// Provides possibilities to customize your device settings.
    /// </summary>
    public static class Devices
    {
        /// <summary>
        /// Changes whether the drive letter is shown before the drive name.
        /// </summary>
        public static bool DriveLetterFirst
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("ShowDriveLetterFirst").ToString()); }
                catch { return false; }
                if (ans == 4)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer", RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (value)
                { regKey.SetValue("ShowDriveLetterFirst", 4, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("ShowDriveLetterFirst", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Changes whether Windows checks the drive's space.
        /// </summary>
        public static bool DiskSpaceCheck
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Policies", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("NoLowDiskSpaceChecks").ToString()); }
                catch { return true; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Policies", 
                    RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (value)
                { regKey.SetValue("NoLowDiskSpaceChecks", 0, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("NoLowDiskSpaceChecks", 1, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Opens the optical drive.
        /// </summary>
        public static void OpenOpticalDrive()
        {
            Win32Api.mciExecute("Set CDAudio door open");
            return;
        }

        /// <summary>
        /// Closes the optical drive.
        /// </summary>
        public static void CloseOpticalDrive()
        {
            Win32Api.mciExecute("Set CDAudio door closed");
            return;
        }
    }
}
